import unittest

from bluecanary.managers import AlarmsManager


class TestAlarmsManager(unittest.TestCase):
    def setUp(self):
        super(TestAlarmsManager, self).setUp()
        self.alarm_group = {
            'AWSProfile': 'fake-profile',
            'TagKey': 'fake-key',
            'TagValue': 'fake-value',
            'Alarms': [
                {
                    'MetricName': 'fake-metric',
                    'Statistic': 'Average',
                    'Period': 60,
                    'EvaluationPeriods': 1,
                    'Threshold': 1,
                    'ComparisonOperator': 'GreaterThanOrEqualToThreshold',
                    'AlarmActions': [
                        'fake-alarm-action',
                    ],
                    'Namespace': 'AWS/EC2',
                },
            ],
        }

    def test_add_alarm_group(self):
        AlarmsManager.add_alarm_group(**self.alarm_group)

        self.assertEqual(AlarmsManager.AlarmGroups.get('fake-key:fake-value'),
                         self.alarm_group)

    def test_get_alarm_group(self):
        AlarmsManager.add_alarm_group(**self.alarm_group)

        self.assertEqual(AlarmsManager.get_alarm_group('fake-key:fake-value'),
                         self.alarm_group)

    def test_get_alarm_keys(self):
        AlarmsManager.add_alarm_group(**self.alarm_group)

        self.assertEqual(list(AlarmsManager.get_alarm_keys()),
                         ['fake-key:fake-value'])



import functools

from bluecanary.managers import AWSCredentialsManager


def preserve_credentials_state(func):
    @functools.wraps(func)
    def manage_credentials_decorator(*args, **kwargs):
        AWSCredentialsManager.save_environment_state()

        func(*args, **kwargs)

        AWSCredentialsManager.load_saved_environment_state()

    return manage_credentials_decorator



import unittest

from bluecanary.plugins import load_plugins, PluginBaseClass


class TestLoadPlugins(unittest.TestCase):
    def test_load_plugins(self):
        plugins = load_plugins()

        self.assertIn('DiskSpaceUtilization', plugins.keys())
        self.assertTrue(isinstance(plugins['DiskSpaceUtilization'], PluginBaseClass))

        self.assertIn('MemoryUtilization', plugins.keys())
        self.assertTrue(isinstance(plugins['MemoryUtilization'], PluginBaseClass))



import os
import unittest
from tempfile import NamedTemporaryFile


from bluecanary.managers import (
    AlarmsManager,
    AWSCredentialsManager,
    AWSProfile,
    ConfigurationManager,
    TagsManager,
)
from bluecanary.utilities.load_yaml import load_yaml_file


class TestLoadYaml(unittest.TestCase):
    def setUp(self):
        super(TestLoadYaml, self).setUp()
        fake_data = (
            b'---\n'
            b'Configuration:\n'
            b'  Plugins:\n'
            b'    - ~/plugins\n'
            b'    - ~/other/plugins\n'
            b'AWSProfiles:\n'
            b'  -\n'
            b'    AWS_PROFILE: fake-profile\n'
            b'    AWS_ACCESS_KEY_ID: fake-access-key-id\n'
            b'    AWS_SECRET_ACCESS_KEY: fake-secret-access-key\n'
            b'    AWS_DEFAULT_REGION: fake-region\n'
            b'AlarmGroups:\n'
            b'  -\n'
            b'    AWSProfile: fake-profile\n'
            b'    TagKey: fake-key\n'
            b'    TagValue: fake-value\n'
            b'    Alarms:\n'
            b'      -\n'
            b'        MetricName: fake-metric\n'
            b'        Statistic: Average\n'
            b'        Period: 60\n'
            b'        EvaluationPeriods: 1\n'
            b'        Threshold: 1\n'
            b'        ComparisonOperator: GreaterThanOrEqualToThreshold\n'
            b'        AlarmActions:\n'
            b'          - fake-alarm-action\n'
            b'        Namespace: AWS/EC2\n'
            b'TagGroups:\n'
            b'  -\n'
            b'    AWSProfile: fake-profile\n'
            b'    TagKey: fake-key\n'
            b'    TagValue: fake-value\n'
            b'    Type: EC2\n'
            b'    Resources:\n'
            b'      - fake-resource\n'
        )

        self.tmp_file = NamedTemporaryFile(delete=False)
        self.tmp_file.write(fake_data)
        self.tmp_file.close()

    def tearDown(self):
        super(TestLoadYaml, self).tearDown()
        os.unlink(self.tmp_file.name)
        ConfigurationManager.Configuration['plugins'] = []

    def test_load_yaml_file_loads_aws_profiles_into_the_AWSCredentialsManager(self):
        expected_profile = AWSProfile(
            AWS_PROFILE='fake-profile',
            AWS_ACCESS_KEY_ID='fake-access-key-id',
            AWS_SECRET_ACCESS_KEY='fake-secret-access-key',
            AWS_DEFAULT_REGION='fake-region',
        )

        load_yaml_file(self.tmp_file.name)

        self.assertEqual(AWSCredentialsManager.AWS_PROFILES.get('fake-profile'),
                         expected_profile)

    def test_load_yaml_file_loads_alarm_groups_into_the_AlarmsManager(self):
        expected_group = {
            'AWSProfile': 'fake-profile',
            'TagKey': 'fake-key',
            'TagValue': 'fake-value',
            'Alarms': [
                {
                    'MetricName': 'fake-metric',
                    'Statistic': 'Average',
                    'Period': 60,
                    'EvaluationPeriods': 1,
                    'Threshold': 1,
                    'ComparisonOperator': 'GreaterThanOrEqualToThreshold',
                    'AlarmActions': [
                        'fake-alarm-action',
                    ],
                    'Namespace': 'AWS/EC2',
                },
            ],
        }

        load_yaml_file(self.tmp_file.name)

        self.assertEqual(AlarmsManager.get_alarm_group('fake-key:fake-value'),
                         expected_group)

    def test_load_yaml_file_loads_alarm_groups_into_the_TagsManager(self):
        expected_group = {
            'AWSProfile': 'fake-profile',
            'TagKey': 'fake-key',
            'TagValue': 'fake-value',
            'Type': 'EC2',
            'Resources': ['fake-resource'],
        }

        load_yaml_file(self.tmp_file.name)

        self.assertEqual(TagsManager.get_tag_group('fake-key', 'fake-value'),
                         expected_group)

    def test_load_yaml_file_loads_plugins_into_the_ConfigurationManager(self):
        expected_plugins = ['~/plugins', '~/other/plugins']

        load_yaml_file(self.tmp_file.name)

        self.assertListEqual(ConfigurationManager.get_plugins_directories(),
                             expected_plugins)



import unittest

from bluecanary.managers import TagsManager


class TestTagsManager(unittest.TestCase):
    def setUp(self):
        super(TestTagsManager, self).setUp()

        self.tag_group = {
            'AWSProfile': 'fake-profile',
            'TagKey': 'fake-key',
            'TagValue': 'fake-value',
            'Type': 'EC2',
            'Resources': ['fake-resource'],
        }

    def test_add_tag_group(self):
        TagsManager.add_tag_group(**self.tag_group)

        self.assertEqual(TagsManager.TagGroups.get('fake-key:fake-value'),
                         self.tag_group)

    def test_get_tag_group(self):
        TagsManager.add_tag_group(**self.tag_group)

        self.assertEqual(TagsManager.get_tag_group('fake-key', 'fake-value'),
                         self.tag_group)

    def test_get_groups_by_type(self):
        TagsManager.add_tag_group(**self.tag_group)

        self.assertEqual([self.tag_group], TagsManager.get_groups_by_type('EC2'))



import click
import boto3

from bluecanary.utilities import throttle


def get_all_ec2_tags(tag_key):
    raw_ec2_tags = get_raw_ec2_tags(tag_key)

    return process_raw_ec2_tags(raw_ec2_tags)


@throttle()
def get_raw_ec2_tags(tag_key):
    ec2_client = boto3.client('ec2')

    return ec2_client.describe_tags(
        Filters=[
            {
                'Name': 'key',
                'Values': [
                    tag_key,
                ]
            },
        ],
    )


def process_raw_ec2_tags(response):
    ec2_tags = dict()

    for ec2 in response.get('Tags'):
        ec2_tags[ec2['ResourceId']] = '{}:{}'.format(ec2['Key'], ec2['Value'])

    return ec2_tags


@throttle()
def set_ec2_tags(instance_ids, tag_key, tag_value, verbose=0):
    if verbose > 1:
        click.echo('Updating the following instances tagged with {}: {}'
                   .format(tag_key, tag_value))
        click.echo(', '.join(instance_ids))
    elif verbose:
        click.echo('Updating instances tagged with {}: {}\n'
                   .format(tag_key, tag_value))

    ec2_client = boto3.client('ec2')

    return ec2_client.create_tags(
        Resources=instance_ids,
        Tags=[
            {
                'Key': tag_key,
                'Value': tag_value,
            },
        ]
    )



import functools
from time import sleep

from botocore.exceptions import ClientError


DECISECOND = 0.1
MAX_RETRIES = 10


def throttle(max_retries=MAX_RETRIES, interval=DECISECOND):
    def throttle_decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):

            retries = 0
            while retries < max_retries:
                try:
                    sleep((2 ^ retries) * interval)
                    return func(*args, **kwargs)
                except ClientError as e:
                    if 'Throttling' in e.args[0]:
                        retries += 1
                    else:
                        raise e

        return wrapper

    return throttle_decorator



import os
import unittest

from bluecanary.managers import AWSProfile, AWSCredentialsManager


class TestAWSCredentialsManager(unittest.TestCase):
    def test_save_environment_state(self):
        os.environ['AWS_PROFILE'] = 'test_profile'
        os.environ['AWS_ACCESS_KEY_ID'] = 'test_id'
        os.environ['AWS_SECRET_ACCESS_KEY'] = 'test_secret'
        os.environ['AWS_DEFAULT_REGION'] = 'test_region'

        AWSCredentialsManager.save_environment_state()

        self.assertDictEqual(AWSCredentialsManager.SAVED_ENVIRONMENT_STATE,
                             {
                                 'AWS_PROFILE': 'test_profile',
                                 'AWS_ACCESS_KEY_ID': 'test_id',
                                 'AWS_SECRET_ACCESS_KEY': 'test_secret',
                                 'AWS_DEFAULT_REGION': 'test_region',
                             })

    def test_clean_current_profile(self):
        os.environ['AWS_PROFILE'] = 'test_profile'
        os.environ['AWS_ACCESS_KEY_ID'] = 'test_id'
        os.environ['AWS_SECRET_ACCESS_KEY'] = 'test_secret'
        os.environ['AWS_DEFAULT_REGION'] = 'test_region'

        AWSCredentialsManager.clean_current_profile()

        self.assertEqual(os.environ.get('AWS_PROFILE'), None)
        self.assertEqual(os.environ.get('AWS_ACCESS_KEY_ID'), None)
        self.assertEqual(os.environ.get('AWS_SECRET_ACCESS_KEY'), None)
        self.assertEqual(os.environ.get('AWS_DEFAULT_REGION'), None)

    def test_load_saved_environment_state(self):
        os.environ['AWS_PROFILE'] = 'test_profile'
        os.environ['AWS_ACCESS_KEY_ID'] = 'test_id'
        os.environ['AWS_SECRET_ACCESS_KEY'] = 'test_secret'
        os.environ['AWS_DEFAULT_REGION'] = 'test_region'

        AWSCredentialsManager.save_environment_state()
        AWSCredentialsManager.clean_current_profile()
        AWSCredentialsManager.load_saved_environment_state()

        self.assertEqual(os.environ.get('AWS_PROFILE'), 'test_profile')
        self.assertEqual(os.environ.get('AWS_ACCESS_KEY_ID'), 'test_id')
        self.assertEqual(os.environ.get('AWS_SECRET_ACCESS_KEY'), 'test_secret')
        self.assertEqual(os.environ.get('AWS_DEFAULT_REGION'), 'test_region')

    def test_set_current_profile(self):
        AWSCredentialsManager.clean_current_profile()

        profile = AWSProfile(
            AWS_PROFILE='test_profile',
            AWS_ACCESS_KEY_ID='test_id',
            AWS_SECRET_ACCESS_KEY='test_secret',
            AWS_DEFAULT_REGION='test_region',
        )

        AWSCredentialsManager.set_current_profile(profile)

        self.assertEqual(os.environ.get('AWS_PROFILE'), None)
        self.assertEqual(os.environ.get('AWS_ACCESS_KEY_ID'), 'test_id')
        self.assertEqual(os.environ.get('AWS_SECRET_ACCESS_KEY'), 'test_secret')
        self.assertEqual(os.environ.get('AWS_DEFAULT_REGION'), 'test_region')

    def test_add_aws_profile(self):
        AWSCredentialsManager.clean_current_profile()

        expected_profile = AWSProfile(
            AWS_PROFILE='test_profile',
            AWS_ACCESS_KEY_ID='test_id',
            AWS_SECRET_ACCESS_KEY='test_secret',
            AWS_DEFAULT_REGION='test_region',
        )

        AWSCredentialsManager.add_aws_profile(
            aws_profile='test_profile',
            aws_access_key_id='test_id',
            aws_secret_access_key='test_secret',
            aws_default_region='test_region',
        )

        self.assertEqual(AWSCredentialsManager.AWS_PROFILES.get('test_profile'),
                         expected_profile)

    def test_load_aws_profile(self):
        AWSCredentialsManager.clean_current_profile()

        AWSCredentialsManager.add_aws_profile(
            aws_profile='test_profile',
            aws_access_key_id='test_id',
            aws_secret_access_key='test_secret',
            aws_default_region='test_region',
        )

        AWSCredentialsManager.load_aws_profile('test_profile')

        self.assertEqual(os.environ.get('AWS_PROFILE'), None)
        self.assertEqual(os.environ.get('AWS_ACCESS_KEY_ID'), 'test_id')
        self.assertEqual(os.environ.get('AWS_SECRET_ACCESS_KEY'), 'test_secret')
        self.assertEqual(os.environ.get('AWS_DEFAULT_REGION'), 'test_region')

        AWSCredentialsManager.load_aws_profile('test_another_profile')

        self.assertEqual(os.environ.get('AWS_PROFILE'), 'test_another_profile')
        self.assertEqual(os.environ.get('AWS_ACCESS_KEY_ID'), None)
        self.assertEqual(os.environ.get('AWS_SECRET_ACCESS_KEY'), None)
        self.assertEqual(os.environ.get('AWS_DEFAULT_REGION'), None)



import unittest

from bluecanary.set_cloudwatch_alarm import set_cloudwatch_alarm
from bluecanary.exceptions import NamespaceError


class TestSetCloudwatchAlarm(unittest.TestCase):

    def setUp(self):

        self.identifier = 'i-12345678'
        self.alarm_kwargs = {
            'MetricName': 'CPUUtilization',
            'Statistic': 'Average',
            'Period': 60,
            'EvaluationPeriods': 5,
            'Threshold': 90,
            'ComparisonOperator': 'GreaterThanOrEqualToThreshold',
            'AlarmActions': ['arn:aws:sns:us-east-1:123456789012:test-sns-action'],
            'Namespace': 'AWS/EC2',
        }

    def test_is_fails_if_an_unknown_namespace_is_provided(self):

        alarm_kwargs = self.alarm_kwargs.copy()
        alarm_kwargs['Namespace'] = 'unknown/namespace'

        with self.assertRaises(NamespaceError) as context_manager:
            set_cloudwatch_alarm(identifier=self.identifier, **alarm_kwargs)

        exception = context_manager.exception
        self.assertEqual(
            exception.args[0],
            'Namespace "unknown/namespace" is not supported by Blue Canary. '
            'If you are using a plugin that supports this Namespace please '
            'ensure that the plugin alarm class does not return None when '
            'calling the "get_dimensions" method.'
        )



class AlarmsManager(object):

    AlarmGroups = {}

    @classmethod
    def add_alarm_group(cls, **kwargs):
        key = '{}:{}'.format(kwargs['TagKey'], kwargs['TagValue'])

        cls.AlarmGroups[key] = kwargs

    @classmethod
    def get_alarm_group(cls, key):
        return cls.AlarmGroups.get(key)

    @classmethod
    def get_alarm_keys(cls):
        return cls.AlarmGroups.keys()



from setuptools import setup, find_packages


setup(
    name='bluecanary',
    version='0.0.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Boto3',
        'Click',
        'coverage',
        'flake8',
        'nose',
        'pluginbase',
        'PyYAML',
    ],
    entry_points='''
        [console_scripts]
        bluecanary=bluecanary.scripts.bluecanary:cli
    ''',
)




from .alarms_manager import AlarmsManager
from .aws_credentials_manager import AWSCredentialsManager, AWSProfile
from .configuration_manager import ConfigurationManager
from .tags_manager import TagsManager



#!/usr/bin/env python
import click

from ..utilities.load_yaml import load_path
from .update_alarms import update_ec2_alarms, update_elb_alarms
from .update_tags import update_ec2_tags, update_elb_tags


@click.group()
def cli():
    pass


@cli.command()
@click.option('--path', '-p', type=click.Path(), multiple=True,
              help='File or Directory path to load yaml files from')
@click.option('--verbose', '-v', count=True,
              help='Enable verbose output')
def update_tags(path, verbose):
    load_path(path)

    if verbose:
        click.echo('\nRunning tags update...\n')

    update_ec2_tags(verbose=verbose)
    update_elb_tags(verbose=verbose)


@cli.command()
@click.option('--path', '-p', type=click.Path(), multiple=True,
              help='File or Directory path to load yaml files from')
@click.option('--verbose', '-v', count=True,
              help='Enable verbose output')
def update_alarms(path, verbose):
    load_path(path)

    if verbose:
        click.echo('\nRunning alarms update...\n')

    update_ec2_alarms(verbose)
    update_elb_alarms(verbose)
